@@ -1,6 +1,7 @@
 import inspect
 import warnings
 import pysb
+import sympy
 
 
 class BngGenerator(object):
@@ -21,17 +22,24 @@ class BngGenerator(object):
         self.generate_parameters()
         self.generate_compartments()
         self.generate_molecule_types()
-        self.generate_reaction_rules()
         self.generate_observables()
+        self.generate_functions()
         self.generate_species()
+        self.generate_reaction_rules()
 
     def generate_parameters(self):
-        if not self.model.parameters:
+        exprs = self.model.expressions_constant()
+        if not self.model.parameters and not exprs:
             return
         self.__content += "begin parameters\n"
-        max_length = max(len(p.name) for p in self.model.parameters)
+        max_length = max(len(p.name) for p in
+                         self.model.parameters | self.model.expressions)
         for p in self.model.parameters:
-            self.__content += ("  %-" + str(max_length) + "s   %e\n") % (p.name, p.value)
+            self.__content += (("  %-" + str(max_length) + "s   %e\n") %
+                               (p.name, p.value))
+        for e in exprs:
+            self.__content += (("  %-" + str(max_length) + "s   %s\n") %
+                               (e.name, sympy_to_muparser(e.expr)))
         self.__content += "end parameters\n\n"
 
     def generate_compartments(self):
@@ -104,6 +112,18 @@ class BngGenerator(object):
                               (obs.match.title(), obs.name, observable_code)
         self.__content += "end observables\n\n"
 
+    def generate_functions(self):
+        exprs = self.model.expressions_dynamic()
+        if not exprs:
+            return
+        max_length = max(len(e.name) for e in exprs) + 2
+        self.__content += "begin functions\n"
+        for i, e in enumerate(exprs):
+            signature = e.name + '()'
+            self.__content += ("  %-" + str(max_length) + "s   %s\n") % \
+                (signature, sympy_to_muparser(e.expr))
+        self.__content += "end functions\n\n"
+
     def generate_species(self):
         if not self.model.initial_conditions:
             warn_caller("Model does not contain any initial conditions")
@@ -116,8 +136,6 @@ class BngGenerator(object):
             self.__content += ("  %-" + str(max_length) + "s   %s\n") % (code, param.name)
         self.__content += "end species\n"
 
-
-
 def format_monomer_site(monomer, site):
     ret = site
     if monomer.site_states.has_key(site):
@@ -191,3 +209,9 @@ def warn_caller(message):
         caller_frame = caller_frame.f_back
         module = inspect.getmodule(caller_frame)
     warnings.warn(message, stacklevel=stacklevel)
+
+def sympy_to_muparser(expr):
+    code = sympy.fcode(expr)
+    code = code.replace('\n     @', '')
+    code = code.replace('**', '^')
+    return code
