@@ -1,3 +1,4 @@
+import pysb.core
 import pysb.bng
 import numpy
 from scipy.integrate import ode
@@ -85,17 +86,30 @@ class Solver(object):
 
         pysb.bng.generate_equations(model)
 
-        code_eqs = '\n'.join(['ydot[%d] = %s;' % (i, sympy.ccode(model.odes[i])) for i in range(len(model.odes))])
-        code_eqs = re.sub(r's(\d+)', lambda m: 'y[%s]' % (int(m.group(1))), code_eqs)
+        code_eqs = '\n'.join(['ydot[%d] = %s;' %
+                              (i, sympy.ccode(model.odes[i]))
+                              for i in range(len(model.odes))])
+        code_eqs = re.sub(r's(\d+)',
+                          lambda m: 'y[%s]' % (int(m.group(1))), code_eqs)
+        for e in model.expressions:
+            code_eqs = re.sub(r'\b(%s)\b' % e.name,
+                              sympy.ccode(e.expand_expr()), code_eqs)
         for i, p in enumerate(model.parameters):
             code_eqs = re.sub(r'\b(%s)\b' % p.name, 'p[%d]' % i, code_eqs)
 
         Solver._test_inline()
-        # If we can't use weave.inline to run the C code, compile it as Python code instead for use with
-        # exec. Note: C code with array indexing, basic math operations, and pow() just happens to also
-        # be valid Python.  If the equations ever have more complex things in them, this might fail.
+        # If we can't use weave.inline to run the C code, compile it as Python
+        # code instead for use with exec. Note: C code with array indexing,
+        # basic math operations, and pow() just happens to also be valid
+        # Python.  If the equations ever have more complex things in them, this
+        # might fail.
         if not Solver._use_inline:
             code_eqs_py = compile(code_eqs, '<%s odes>' % model.name, 'exec')
+        else:
+            for arr_name in ('ydot', 'y', 'p'):
+                macro = arr_name.upper() + '1'
+                code_eqs = re.sub(r'\b%s\[(\d+)\]' % arr_name,
+                                  '%s(\\1)' % macro, code_eqs)
 
         def rhs(t, y, p):
             ydot = self.ydot
@@ -106,7 +120,8 @@ class Solver(object):
                 exec code_eqs_py in locals()
             return ydot
 
-        # build integrator options list from our defaults and any kwargs passed to this function
+        # build integrator options list from our defaults and any kwargs passed
+        # to this function
         options = {}
         try:
             options.update(default_integrator_options[integrator])
@@ -123,6 +138,12 @@ class Solver(object):
                                                       itertools.repeat(float)))
         else:
             self.yobs = numpy.ndarray((len(tspan), 0))
+        exprs = model.expressions_dynamic()
+        if len(exprs):
+            self.yexpr = numpy.ndarray(len(tspan), zip(exprs.keys(),
+                                                       itertools.repeat(float)))
+        else:
+            self.yexpr = numpy.ndarray((len(tspan), 0))
         self.yobs_view = self.yobs.view(float).reshape(len(self.yobs), -1)
         self.integrator = ode(rhs).set_integrator(integrator, **options)
 
@@ -150,13 +171,16 @@ class Solver(object):
         if param_values is not None:
             # accept vector of parameter values as an argument
             if len(param_values) != len(self.model.parameters):
-                raise Exception("param_values must be the same length as model.parameters")
+                raise Exception("param_values must be the same length as "
+                                "model.parameters")
             if not isinstance(param_values, numpy.ndarray):
                 param_values = numpy.array(param_values)
         else:
             # create parameter vector from the values in the model
             param_values = numpy.array([p.value for p in self.model.parameters])
 
+        subs = dict((p, param_values[i])
+                    for i, p in enumerate(self.model.parameters))
         if y0 is not None:
             # accept vector of species amounts as an argument
             if len(y0) != self.y.shape[1]:
@@ -165,19 +189,26 @@ class Solver(object):
                 y0 = numpy.array(y0)
         else:
             y0 = numpy.zeros((self.y.shape[1],))
-            for cp, ic_param in self.model.initial_conditions:
-                pi = self.model.parameters.index(ic_param)
+            for cp, value_obj in self.model.initial_conditions:
+                if value_obj in self.model.parameters:
+                    pi = self.model.parameters.index(value_obj)
+                    value = param_values[pi]
+                elif value_obj in self.model.expressions:
+                    value = value_obj.expand_expr().evalf(subs=subs)
+                else:
+                    raise ValueError("Unexpected initial condition value type")
                 si = self.model.get_species_index(cp)
                 if si is None:
                     raise Exception("Species not found in model: %s" % repr(cp))
-                y0[si] = param_values[pi]
+                y0[si] = value
 
         # perform the actual integration
         self.integrator.set_initial_value(y0, self.tspan[0])
         self.integrator.set_f_params(param_values)
         self.y[0] = y0
         i = 1
-        while self.integrator.successful() and self.integrator.t < self.tspan[-1]:
+        while (self.integrator.successful() and
+               self.integrator.t < self.tspan[-1]):
             self.y[i] = self.integrator.integrate(self.tspan[i])
             i += 1
         if self.integrator.t < self.tspan[-1]:
@@ -186,6 +217,12 @@ class Solver(object):
         for i, obs in enumerate(self.model.observables):
             self.yobs_view[:, i] = \
                 (self.y[:, obs.species] * obs.coefficients).sum(1)
+        obs_names = self.model.observables.keys()
+        obs_dict = dict((k, self.yobs[k]) for k in obs_names)
+        for expr in self.model.expressions_dynamic():
+            expr_subs = expr.expand_expr().subs(subs)
+            func = sympy.lambdify(obs_names, expr_subs, "numpy")
+            self.yexpr[expr.name] = func(**obs_dict)
 
 
 def odesolve(model, tspan, param_values=None, y0=None, integrator='vode',
@@ -281,7 +318,7 @@ def odesolve(model, tspan, param_values=None, y0=None, integrator='vode',
     >>> print yfull[0:4, 1:3]
     Traceback (most recent call last):
       ...
-    IndexError: too many indices
+    IndexError: too many indices for array
     >>> yarray = yfull.view(float).reshape(len(yfull), -1)
     >>> print yarray.shape
     (10, 6)
@@ -309,3 +346,14 @@ def odesolve(model, tspan, param_values=None, y0=None, integrator='vode',
     yfull_view[:, solver.y.shape[1]:] = solver.yobs_view
 
     return yfull
+
+
+def setup_module(module):
+    """Doctest fixture for nose."""
+    # Distutils' temp directory creation code has a more-or-less unsuppressable
+    # print to stdout which will break the doctest which triggers it (via
+    # scipy.weave.inline). So here we run an end-to-end test of the inlining
+    # system to get that print out of the way at a point where it won't matter.
+    # As a bonus, the test harness is suppressing writes to stdout at this time
+    # anyway so the message is just swallowed silently.
+    Solver._test_inline()
