@@ -23,13 +23,22 @@ def rules_using_parameter(model, parameter):
     return cset
 
 
-def synthetic_data(model, tspan, obs_list=None, sigma=0.1, seed=None):
-    from pysb.integrate import odesolve
-    random = numpy.random.RandomState(seed)
-    ysim = odesolve(model, tspan)
-    ysim_array = ysim.view().reshape(len(tspan), len(ysim.dtype))
-    ysim_array *= (random.randn(*ysim_array.shape) * sigma + 1);
-    return ysim
+def synthetic_data(model, tspan, obs_list=None, sigma=0.1):
+    #from pysb.integrate import odesolve
+    from pysb.integrate import Solver
+    solver = Solver(model, tspan)
+    solver.run()
+
+    # Sample from a normal distribution with variance sigma and mean 1
+    # (randn generates a matrix of random numbers sampled from a normal
+    # distribution with mean 0 and variance 1)
+    # 
+    # Note: This modifies yobs_view (the view on yobs) so that the changes 
+    # are reflected in yobs (which is returned by the function). Since a new
+    # Solver object is constructed for each function invocation this does not
+    # cause problems in this case.
+    solver.yobs_view *= ((numpy.random.randn(*solver.yobs_view.shape) * sigma) + 1)
+    return solver.yobs
 
 def get_param_num(model, name):
     for i in range(len(model.parameters)):
